#include <GUIConstants.au3>
#include <WindowsConstants.au3>

#include "globals.au3"
#include "json.au3"

#include "gui_functions.au3"



; #FUNCTION# ====================================================================================================================
; Name ..........: _GUICreate_FromXML
; Description ...:
; Syntax ........: _GUICreate_FromXML(Byref $oGUI, $sPath)
; Parameters ....: $oGUI                - [in/out] Object to store handles in.
;                  $sPath               - Path to XML to load gui from.
; Return values .: 1 if GUI was created 0 if not
;                  @error 1: file does not exist
;                         2: loaded file has json error
;                         3: error creating control
;                         4: error creating subform
; ===============================================================================================================================
Func _GUICreate_FromXML(ByRef $oGUI, $sPath)
	If Not FileExists($sPath) Then Return SetError(1, 0, 0)

	Local $sFileData = FileRead($sPath)
	$oGUI = _JSON_Parse($sFileData)

	If @error Then Return SetError(2, @error, 0)
	;If $bDebug Then ConsoleWrite(_JSON_Generate($oGUI) & @CRLF)

	If $oGUI.Item("active") == "true" Then
		Opt("GUIOnEventMode", 1)

		;Create GUI
		With $oGUI.Item("dimensions")
			$oGUI.Item("hwnd") = GUICreate(	$oGUI.Item("title"), _
											.Item("width"), _
											.Item("height"), _
											.Item("left"), _
											.Item("top"))
		EndWith
		GUISetBkColor(0x0000FF, HWnd($oGUI.item("hwnd")))

		;register guievents
		If $oGUI.Exists("onclose") Then
			If $oGUI.Item("onclose") <> "" Then
				Local $iSetOnEvent = GUISetOnEvent($GUI_EVENT_CLOSE, $oGUI.Item("onclose"), HWnd($oGUI.Item("hwnd")))
				ConsoleWrite("@@ Debug line" & @TAB & @ScriptLineNumber & "  _GUICreate_FromXML: var: $iSetOnEvent --> " & $iSetOnEvent & ' HWnd($oGUI.Item("hwnd")) = ' & HWnd($oGUI.Item("hwnd")) & @CRLF)
			EndIf
		EndIf

		;create the controls for the main gui (if any)
		For $i = 0 To UBound($oGUI.Item("controls")) - 1
			;check (for safety) if the datastructure is correct
			If IsObj($oGUI.Item("controls")[$i]) Then
				;create the control in a seperate function for cleaner code
				Local $oControl = $oGUI.Item("controls")[$i]
				__GUICreate_Control($oControl)

				;errors?
				If @error Then Return SetError(3, 0, 0)
			EndIf
		Next

		;now create the subforms and load the standard one
		For $i = 0 To UBound($oGUI.Item("subforms")) - 1
			;check (for safety) if the datastructure is correct
			If IsObj($oGUI.Item("subforms")[$i]) Then
				;create the subform in a seperate function for cleaner code
				Local $oSubform = $oGUI.Item("subforms")[$i]
				__GUICreate_Subform($oSubform, $oGUI.Item("hwnd"))

				;errors?
				If @error Then Return SetError(4, 0, 0)
			EndIf
		Next

		;show GUI, so the user can see it...
		GUISetState(@SW_SHOW, HWnd($oGUI.Item("hwnd")))
	Else
		ConsoleWrite($sPath & " | This File is deactivated! Skipping..." & @CRLF)
	EndIf

	Return True
EndFunc  ;==>_GUICreate_FromXML

Func __GUICreate_Control(ByRef $oObj)
	;what type of control should we create...
	With $oObj.Item("body")
		Switch .Item("type")
			Case "button"
				.Item("hwnd") = GUICtrlCreateButton(.Item("title"), _
													.Item("dimensions").Item("left"), _
													.Item("dimensions").Item("top"), _
													.Item("dimensions").Item("width"), _
													.Item("dimensions").Item("height"))
				ConsoleWrite("@@ Debug line" & @TAB & @ScriptLineNumber & "  __GUICreate_Control: var: .Item('hwnd') --> " & .Item("hwnd") & @CRLF)
				ConsoleWrite("@@ Debug line" & @TAB & @ScriptLineNumber & "  __GUICreate_Control: var: .Item('title') --> " & .Item("title") & @CRLF)
				ConsoleWrite("@@ Debug line" & @TAB & @ScriptLineNumber & "  __GUICreate_Control: var: .Item('dimensions').Item('left')   --> " & .Item('dimensions').Item('left')   & @CRLF)
				ConsoleWrite("@@ Debug line" & @TAB & @ScriptLineNumber & "  __GUICreate_Control: var: .Item('dimensions').Item('top')    --> " & .Item('dimensions').Item('top')    & @CRLF)
				ConsoleWrite("@@ Debug line" & @TAB & @ScriptLineNumber & "  __GUICreate_Control: var: .Item('dimensions').Item('width')  --> " & .Item('dimensions').Item('width')  & @CRLF)
				ConsoleWrite("@@ Debug line" & @TAB & @ScriptLineNumber & "  __GUICreate_Control: var: .Item('dimensions').Item('height') --> " & .Item('dimensions').Item('height') & @CRLF)
				;set onclick events for controls (if exists)
				If .Exists("onclick") Then
					If .Item("onclick") <> "" Then
						Local $iSetOnEvent = GUICtrlSetOnEvent(.Item("hwnd"), .Item("onclick"))
						ConsoleWrite("@@ Debug line" & @TAB & @ScriptLineNumber & "  __GUICreate_Control: var: $iSetOnEvent --> " & $iSetOnEvent & '  .Item("hwnd") = ' & .Item("hwnd") & '  .Item("onclick") = ' & .Item("onclick") & @CRLF)
					Else
						ConsoleWrite('! __GUICreate_Control: .Item("onclick") = ""' & @CRLF)
					EndIf
				Else
					ConsoleWrite('! __GUICreate_Control: .Exists("onclick") = False' & @CRLF)
				EndIf

				;set control active (if exists)
				If .Exists("active") Then
					If Not (.Item("active") == "true") Then GUICtrlSetState(.Item("hwnd"), $GUI_DISABLE)
				EndIf
		EndSwitch
	EndWith

	Return SetError(0, 0, 1)
EndFunc  ;==>__GUICreate_Control

Func __GUICreate_Subform(ByRef $oObj, $hParent)
	;what type of control should we create...
	With $oObj.Item("body")
		If .Exists("active") Then
			If .Item("active") == "true" Then
				If $bDebug Then ConsoleWrite("Create Subform: " & $oObj.Item("id") & @CRLF)

				Switch .Item("type")
					Case "form"
;~ 						GUISwitch(HWnd($hParent))
						; ********************************************************************************************************************
						; GUICreate ( "title" [, width [, height [, left = -1 [, top = -1 [, style = -1 [, exStyle = -1 [, parent = 0]]]]]]] )
						; ********************************************************************************************************************
;~ 						.Item("hwnd") = GUICreate(	.Item("title"), _
;~ 													.Item("dimensions").Item("left"), _
;~ 													.Item("dimensions").Item("top"), _
;~ 													.Item("dimensions").Item("width"), _
;~ 													.Item("dimensions").Item("height"), _
;~ 													$WS_POPUP, _
;~ 													$WS_EX_MDICHILD, _
;~ 													HWnd($hParent))
						.Item("hwnd") = GUICreate(	.Item("title"), _
													.Item("dimensions").Item("width"), _
													.Item("dimensions").Item("height"), _
													.Item("dimensions").Item("left"), _
													.Item("dimensions").Item("top"), _
													$WS_POPUP, _
													$WS_EX_MDICHILD, _
													HWnd($hParent))

						ConsoleWrite("hwnd: " & HWnd(.item("hwnd")) & ' .Item("title") = ' & .Item("title") & ' HWnd($hParent) = ' & HWnd($hParent) & @CRLF)
						Switch .Item("title")
							Case 'test1'
								ConsoleWrite('GUISetBkColor(0x00FF00, '&HWnd(.item("hwnd"))&') = ' & GUISetBkColor(0x00FF00, HWnd(.item("hwnd"))) & @CRLF)
							Case 'test2'
								ConsoleWrite('GUISetBkColor(0xFF0000, '&HWnd(.item("hwnd"))&') = ' & GUISetBkColor(0xFF0000, HWnd(.item("hwnd"))) & @CRLF)
						EndSwitch

						;create the controls for the subform (if any)
						For $i = 0 To UBound(.Item("controls")) - 1
							;check (for safety) if the datastructure is correct
							If IsObj(.Item("controls")[$i]) Then
								;create the control in a seperate function for cleaner code
								Local $oControl = .Item("controls")[$i]
								__GUICreate_Control($oControl)

								;errors?
								If @error Then Return SetError(2, 0, 0)
							EndIf
						Next

;~ 						DllCall("user32.dll", "int", "SetParent", "hwnd", HWnd(.Item("hwnd")), "hwnd", HWnd($hParent))
;~ 						GUISetState(@SW_HIDE, HWnd(.Item("hwnd")))
				EndSwitch
			EndIf
		EndIf
	EndWith
EndFunc  ;==>__GUICreate_Subform